<?php
  /**
   * Třída zpracovává uživatelský vstup týkající správy typů bezpečnostních incidentů
   * (a šablon odpovědí pro e-maily), na základě kterého volá odpovídající metody,
   * přičemž svůj výstup předává další vrstvě pro výpis.
   *
   * @author Martin Šebela
   */
  class IncidentTypesController extends Controller {
    /**
     * Zpracuje vstup z URL adresy a na základě toho zavolá odpovídající metodu.
     *
     * @param array $arguments         Uživatelský vstup
     */
    public function process($arguments) {
      $this->setView('header-types-incidents', true);
      $this->setUrlSection('types-templates');

      $model = new IncidentTypeModel();
      $formData = [
        'inputsNames' => ['name', 'description', 'color'],
        'formPrefix' => 'incident-type-',
        'dbTable' => 'blck_incidents_types'
      ];

      if (isset($_GET['action'])) {
        $id = isset($_GET['id']) ? get_int_from_string($_GET['id']) : false;

        if ($_GET['action'] == ACT_NEW) {
          $this->processNew($model, $formData);
        }
        elseif ($_GET['action'] == ACT_EDIT && $id !== false) {
          $this->processEdit($model, $formData, $id);
        }
        elseif ($_GET['action'] == 'templates' && $id !== false) {
          $this->processTemplates($model, $formData, $id);
        }
        elseif ($_GET['action'] == ACT_DEL && $id !== false) {
          $this->processDelete($model, $id);
        }
        else {
          $this->addMessage(MSG_ERROR, 'Zvolená akce neexistuje.');
          $this->redirect($this->urlSection);
        }
      }
      else {
        $this->processList($model);
      }

      // Odkaz na nápovědu.
      $this->setHelpLink('');
    }


    /**
     * Vypíše formulář a obslouží s ním související prvky pro přidání nového typu bezpečnostního incidentu.
     *
     * @param IncidentTypeModel $model      Instance třídy
     * @param array $formData               Nastavení a vlastnosti formuláře
     */
    private function processNew($model, $formData) {
      $this->setTitle('Nový typ incidentu');
      $this->setView('form-incident-type');

      $model->initForm($formData['inputsNames'], $formData['formPrefix'], $formData['dbTable']);
      $this->initViewData($model, ACT_NEW, $formData['formPrefix']);

      if (isset($_POST[$model->formPrefix . $this->getData('action')])) {
        try {
          $model->load($_POST);

          $model->validateData();
          $model->insertIncidentType();

          $this->addMessage(MSG_SUCCESS, 'Přidání proběhlo úspěšně.');
          $this->redirect($this->urlSection);
        }
        catch (UserError $error) {
          $this->addMessage($error->getCode(), $error->getMessage());
        }
      }
    }


    /**
     * Vypíše formulář a obslouží s ním související prvky pro úpravu konkrétního typu bezpečnostního incidentu.
     *
     * @param IncidentTypeModel $model      Instance třídy
     * @param array $formData               Nastavení a vlastnosti formuláře
     * @param int $idIncidentType           ID typu incidentu
     */
    private function processEdit($model, $formData, $idIncidentType) {
      $this->setTitle('Úprava typu incidentu');
      $this->setView('form-incident-type');

      $model->initForm($formData['inputsNames'], $formData['formPrefix'], $formData['dbTable']);
      $this->setViewData('incidentType', $model->getIncidentType($idIncidentType));

      /* Ověření existence záznamu. */
      $this->checkRecordExistence($this->getData('incidentType'));

      $this->initViewData($model, ACT_EDIT, $formData['formPrefix']);

      if (isset($_POST[$model->formPrefix . $this->getData('action')])) {
        try {
          $model->load($_POST);

          $model->validateData();
          $model->updateIncidentType($idIncidentType);

          $this->addMessage(MSG_SUCCESS, 'Úprava proběhla úspěšně.');
          $this->redirect($this->urlSection);
        }
        catch (UserError $error) {
          $this->addMessage($error->getCode(), $error->getMessage());
        }
      }
    }


    /**
     * Zavolá metodu pro výpis a úpravu šablon pro e-maily u existujících typů bezpečnostních incidentů.
     *
     * @param IncidentTypeModel $model      Instance třídy
     * @param array $formData               Nastavení a vlastnosti formuláře
     * @param int $idIncidentType           ID typu incidentu
     */
    private function processTemplates($model, $formData, $idIncidentType) {
      $this->setTitle('Šablony e-mailů');
      $this->setView('form-email-templates');

      $this->setViewData('incidentType', $model->getIncidentType($idIncidentType));

      // Ověření existence záznamu.
      $this->checkRecordExistence($this->getData('incidentType'));

      $modelTemplates = new EmailTemplatesModel();

      $formData = [
        'inputsNames' => ['subject', 'body'],
        'formPrefix' => 'template-',
        'dbTable' => 'blck_email_templates'
      ];

      $modelTemplates->initForm($formData['inputsNames'], $formData['formPrefix'], $formData['dbTable']);
      $this->initViewData($modelTemplates, 'templates', $formData['formPrefix']);

      // Získání seznamu šablon pro e-maily.
      $emailTemplates = $modelTemplates->getEmailTemplatesByType($idIncidentType);
      $this->setViewData('emailTemplates', $emailTemplates);

      // Úprava konkrétní šablony.
      if (isset($_POST[$modelTemplates->formPrefix . ACT_EDIT])) {
        // Změna prefixu formuláře kvůli více formulářům na jedné stránce.
        $modelTemplates->formPrefix = $modelTemplates->formPrefix . ACT_EDIT . '-';

        $emailTemplate = $modelTemplates->getEmailTemplate(
          (isset($_POST[$modelTemplates->formPrefix . 'id'])) ? $_POST[$modelTemplates->formPrefix . 'id'] : 0
        );

        if (!empty($emailTemplate)) {
          try {
            $modelTemplates->load($_POST);

            $modelTemplates->validateData();
            $modelTemplates->updateEmailTemplate($_POST[$modelTemplates->formPrefix . 'id']);

            $this->addMessage(MSG_SUCCESS, 'Úprava proběhla úspěšně.');
            $this->redirect($this->urlSection . '/' . $this->getData('action') . '/' . $idIncidentType);
          }
          catch (UserError $error) {
            $this->addMessage($error->getCode(), $error->getMessage());
          }
        }
        else {
          $this->addMessage(MSG_ERROR, 'Zvolený záznam neexistuje.');
        }
      }
    }


    /**
     * Zavolá metodu pro odstranění konkrétního typu bezpečnostního incidentu.
     *
     * @param IncidentTypeModel $model     Instance třídy
     * @param int $idIncidentType          ID typu incidentu
     */
    private function processDelete($model, $idIncidentType) {
      if (isset($_POST)) {
        try {
          $model->isValidCsrfToken($_POST);
          $model->deleteIncidentType($idIncidentType);

          $this->addMessage(MSG_SUCCESS, 'Smazání proběhlo úspěšně.');
        }
        catch (UserError $error) {
          $this->addMessage($error->getCode(), $error->getMessage());
        }

        $this->redirect($this->urlSection);
      }
    }


    /**
     * Vypíše seznam typů bezpečnostních incidentů.
     *
     * @param IncidentTypeModel $model      Instance třídy
     */
    private function processList($model) {
      $this->setTitle('Seznam typů incidentů');
      $this->setView('list-types-incidents');

      $this->setViewData('incidentTypes', $model->getIncidentTypes());
    }
  }
