<?php
  /*
   * SOUBOR OBSAHUJÍCÍ OBECNÉ FUNKCE POUŽÍVANÉ NAPŘÍČ ZDROJOVÝM KÓDEM.
   *
   * @author Martin Šebela
   */


  /**
   * Automaticky importuje PHP třídy, které aplikace právě vyžaduje.
   *
   * @param string $className          Název třídy.
   */
  function autoload_functions($className) {
    if (preg_match('/Controller$/', $className)) {
      require CORE_DOCUMENT_ROOT . '/' . CORE_DIR_CONTROLLERS . '/' . $className . '.php';
    }
    elseif (preg_match('/Blocker$/', $className) || $className == 'BlockedIdentity') {
      require CORE_DOCUMENT_ROOT . '/' . CORE_DIR_BLOCKERS . '/' . $className . '.php';
    }
    else {
      require CORE_DOCUMENT_ROOT . '/' . CORE_DIR_MODELS . '/' . $className . '.php';
    }
  }


  /**
   * Vrátí instanci Blockátoru, který na základě IP adresy dokáže
   * prohledat logy a zařízení/uživatele zablokovat/odblokovat.
   *
   * @param string $ipAddress          IP adresa
   * @return mixed|null                Instance Blockátoru
   */
  function get_blocker_by_hostname($ipAddress) {
    $selectedBlocker = null;

    // Seznam existujících modulů Blockátorů, které je možno použít.
    $blockers = [
      new EduroamBlocker(),
      new VPNBlocker(),
      new WiredBlocker()
    ];

    $hostname = gethostbyaddr($ipAddress);

    // Ověření, zdali hostname spadá do kompetence konkrétního modulu (Blockátoru).
    foreach ($blockers as $blockerClass) {
      if ($blockerClass->isFromThisNetwork($hostname)) {
        $selectedBlocker = $blockerClass;
        break;
      }
    }

    return $selectedBlocker;
  }


  /**
   * Provede akci v systému, který je chráněn SSO (Single Sign-On).
   * Autentizace u SSO proběhne pomocí X.509.
   *
   * @param string $url                URL adresa
   * @param null|array $data           Data formou pole (klíč, hodnota), která se mají předat
   *                                   do systému nebo NULL (výchozí).
   * @return bool|string|null          Výsledek akce (obsah stránky)
   * @throws UserError                 Výjimka obsahující textovou informaci o chybě pro uživatele.
   */
  function process_sso($url, $data = null) {
    if (!file_exists(SSO_FILEPATH_PRIVATE_KEY) || !file_exists(SSO_FILEPATH_PUBLIC_KEY)) {
      throw new UserError('Nepodařilo se načíst soubory certifikátu pro autentizaci přes SSO!', MSG_ERROR);
    }

    $ch = curl_init();

    curl_setopt($ch, CURLOPT_SSL_CIPHER_LIST, 'DEFAULT@SECLEVEL=1');
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, TRUE);
    curl_setopt($ch, CURLOPT_VERBOSE, 0);
    curl_setopt($ch, CURLOPT_HEADER, 0);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

    curl_setopt($ch, CURLOPT_SSLKEY, SSO_FILEPATH_PRIVATE_KEY);
    curl_setopt($ch, CURLOPT_SSLCERT, SSO_FILEPATH_PUBLIC_KEY);

    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
    curl_setopt($ch, CURLOPT_MAXREDIRS, 5);
    curl_setopt($ch, CURLOPT_TIMEOUT, 60);

    $cookie_file = tempnam("/tmp", "CURLCOOKIE");
    curl_setopt($ch, CURLOPT_COOKIEJAR, $cookie_file);

    // Předat v URL i potřebná data k zaslání.
    if ($data != null) {
      curl_setopt($ch, CURLOPT_URL, $url . '?' . http_build_query($data));
    }
    else {
      curl_setopt($ch, CURLOPT_URL, $url);
    }

    $content = curl_exec($ch);

    // Změna redirectu po úspěšném přihlášení do SSO, které WebAuth provádí formou meta tagu v HTML.
    $url = preg_replace('/.*META http-equiv="refresh" content="1; url=(.*?)".*/s', "$1", html_entity_decode($content));

    // Výstup SSO (pro debug).
    //var_dump(curl_errno($ch) > 0 ? array('curl_error_' . curl_errno($ch) => curl_error($ch)) : curl_getinfo($ch));

    curl_setopt($ch, CURLOPT_URL, $url);
    $result = curl_exec($ch);

    curl_close($ch);
    unlink($cookie_file);

    return $result;
  }


  /**
   * Nastaví skriptu zabezpečené HTTP hlavičky (viz informace např. na https://securityheaders.io).
   */
  function init_http_security_headers() {
    header('X-Frame-Options: DENY');
    header('Strict-Transport-Security: max-age=31536000; includeSubDomains; preload');
    header('X-Content-Type-Options: nosniff');
    header('Referrer-Policy: strict-origin-when-cross-origin');
    header("Content-Security-Policy: default-src 'self'; upgrade-insecure-requests; script-src 'self' 'unsafe-inline'; font-src 'self'; style-src 'self' 'unsafe-inline'; img-src 'self' data:");
  }


  /**
   * Nastaví a spustí zabezpečnou formu SESSIONS (resp. COOKIES).
   */
  function init_secure_session_start() {
    ini_set('session.cookie_secure', true);
    ini_set('session.cookie_httponly', true);

    session_name('__Host-PHPSESSID');
    session_start();
  }


  /**
   * Vrátí IP adresu uživatele.
   *
   * @return string|null             IP adresa uživatele.
   */
  function get_client_ip() {
    $ipAddress = null;

    if (getenv('HTTP_CLIENT_IP')) {
      $ipAddress = getenv('HTTP_CLIENT_IP');
    }
    elseif (getenv('HTTP_X_FORWARDED')) {
      $ipAddress = getenv('HTTP_X_FORWARDED');
    }
    elseif (getenv('HTTP_FORWARDED_FOR')) {
      $ipAddress = getenv('HTTP_FORWARDED_FOR');
    }
    elseif (getenv('HTTP_FORWARDED')) {
      $ipAddress = getenv('HTTP_FORWARDED');
    }
    elseif (getenv('REMOTE_ADDR')) {
      $ipAddress = getenv('REMOTE_ADDR');
    }

    return $ipAddress;
  }


  /**
   * Rozdělí e-mailovou adresu na části a vrátí tu část, která je požadována.
   *
   * @param string $email              Zkoumaný e-mail.
   * @param null|string $requiredPart  Název části, která má být vrácena ("username" nebo "domain"),
   *                                   nebo NULL (tj. všechny části).
   * @return array|string|null         Zvolená část (nebo části) e-mailu.
   */
  function get_email_part($email, $requiredPart = null) {
    $email = filter_var(trim($email), FILTER_SANITIZE_EMAIL);
    $emailParts = explode('@', $email);

    if ($requiredPart == 'username') {
      return (isset($emailParts[0])) ? $emailParts[0] : null;
    }
    elseif ($requiredPart == 'domain') {
      return (isset($emailParts[1])) ? $emailParts[1] : null;
    }

    return $emailParts;
  }


  /**
   * Vrátí informaci o tom, zdali je uživatel z dané organizace.
   *
   * @param string $email              Identita uživatele (typicky e-mail).
   * @return bool                      TRUE v případě, že je uživatel z dané organizace, jinak FALSE.
   */
  function is_user_from_organization($email) {
    $email = mb_strtolower($email);

    // Pokud identita obsahuje doménu organizace (včetně zavináče) nebo pokud
    // naopak neobsahuje znak zavináče, pak se jedná o uživatele z organizace.
    return strpos($email, mb_strtolower(ORG_DOMAIN)) !== false || strpos($email, '@') === false;
  }


  /**
   * Ověří, zdali je e-mail (resp. uživatelské jméno) validní a neobsahuje speciální, nepovolené znaky.
   *
   * @param string $email              E-mail uživatele nebo uživatelské jméno.
   * @return bool                      TRUE pokud se jedná o validní e-mail nebo uživatelské jméno.
   * @throws UserError                 Výjimka obsahující textovou informaci o chybě pro uživatele.
   */
  function is_valid_username($email) {
    if (strpos($email, '@') !== false) {
      $email = get_email_part($email, 'username');
    }

    if (!preg_match('/^[a-zA-Z0-9._]+$/', $email)) {
      throw new UserError('Uživatelské jméno blokovaného uživatele obsahuje speciální, nepovolené znaky!', MSG_ERROR);
    }

    return true;
  }


  /**
   * Ověří, zdali je předaný vstup číslo a pokud ano, vrátí jej.
   *
   * @param string $string             Testovaný vstup.
   * @return bool|int                  Číslo nebo FALSE, pokud se nejedná o číslo.
   */
  function get_int_from_string($string) {
    if (isset($string) && is_numeric($string)) {
      return $string;
    }
    else {
      return false;
    }
  }


  /**
   * Zformátuje číslo podle zvyklostí v České republice (oddělení jednotlivých řádů apod.).
   *
   * @param int $number                Číslo, které má být zformátováno.
   * @return string                    Zformátované číslo.
   */
  function get_formatted_number($number) {
    return number_format($number,0, ',', ' ');
  }


  /**
   * Zformátuje datum v odlišném formátu a případně jej převede do nastavené časové zóny.
   *
   * @param string $datetime           Datum ke zformátování
   * @return string                    Zformátované a převedené datum
   * @throws UserError                 Výjimka obsahující textovou informaci o chybě pro uživatele.
   */
  function get_formatted_datetime($datetime) {
    // Pokud v datu chybí rok, tak jej zkusit doplnit (na základě CESNET hlášení).
    if (date_parse($datetime)['year'] == false) {
      $date = DateTime::createFromFormat('d.m. H:i:s', $datetime);
    }
    else {
      try {
        $date = new DateTime($datetime);
      }
      catch (Exception $e) {
        throw new UserError('Zadané datum je v nepodporovaném formátu!', MSG_ERROR);
      }
    }

    if ($date == null) {
      throw new UserError('Zadané datum je v nepodporovaném formátu!', MSG_ERROR);
    }

    $time_zone = new DateTimeZone(LOG_TIME_ZONE);
    $date->setTimezone($time_zone);

    return $date->format('Y-m-d H:i:s');
  }


  /**
   * Vrátí jednotlivé části data (rok, měsíc, den) formou pole.
   *
   * @param string $datetime           Datum a čas
   * @return array                     Pole s jednotlivými částmi data
   */
  function get_date_parts($datetime) {
    $date = DateTime::createFromFormat('Y-m-d H:i:s', $datetime);

    return [
      'year' => $date->format('Y'),
      'month' => $date->format('m'),
      'day' => $date->format('d')
    ];
  }


  /**
   * Vrátí prvky pole jako řetězec, kde jsou jednotlivé, původní prvky, oddělené symbolem nového řádku.
   *
   * @param array $array               Pole, které se má převést.
   * @return string                    Prvky pole vrácené jako řetězec oddělený novými řádky.
   */
  function get_array_items_as_newline_string($array) {
    return implode("\n", $array);
  }


  /**
   * Vrátí pole bez duplicit včetně opravy indexování v poli tak,
   * aby klíče postupovaly jednotlivě za sebou.
   *
   * @param array $array               Pole, v němž se mají odstranit duplicity.
   * @return array                     Pole bez duplicit s opravením indexováním.
   */
  function remove_duplicates_in_array($array) {
    return array_values(array_unique($array));
  }