<?php
  /**
   * Třída sloužící k získávání informací o šablonách odpovědí pro e-maily, k přidávání
   * nových šablon, k úpravě těch existujících a k dalším souvisejícím operacím.
   *
   * @author Martin Šebela
   */
  class EmailTemplatesModel extends FormModel {
    /**
     * @var string      Název tabulky v databázi, se kterou je instance svázána.
     */
    private $dbTableName = 'blck_email_templates';

    /**
     * @var string      ID typu incidentu.
     */
    protected $idIncidentType;

    /**
     * @var string      ID status incidentu.
     */
    protected $idIncidentStatus;

    /**
     * @var string      Předmět zprávy.
     */
    protected $subject;

    /**
     * @var string      Obsah zprávy.
     */
    protected $body;

    /**
     * @var string      Jazyková mutace.
     */
    protected $lang;


    /**
     * Připraví novou šablonu v závislosti na vyplněných datech a vrátí ji formou pole.
     *
     * @return array                   Pole obsahující data o šabloně.
     */
    public function makeEmailTemplate() {
      return [
        'subject' => $this->subject,
        'body' => $this->body,
      ];
    }


    /**
     * Uloží do instance a zároveň vrátí (z databáze) informace o zvolené šabloně.
     *
     * @param int $id                  ID šablony
     * @return array                   Pole obsahující informace o šabloně.
     */
    public function getEmailTemplate($id) {
      $this->dbRecordData = Database::querySingle('
              SELECT `id_incident_type`, `id_incident_status`, `id_email_lang`, `subject`, `body`,
              `lang`, `symbol`
              FROM `blck_email_templates`
              JOIN `blck_email_templates_langs`
              ON blck_email_templates.id_email_lang = blck_email_templates_langs.id_lang
              WHERE `id_email_template` = ?'
      , $id);

      return $this->dbRecordData;
    }


    /**
     * Vrátí všechny šablony pro konkrétní typ incidentu.
     *
     * @param int $idType              ID typu bezpečnostního incidentu.
     * @return array                   Pole šablon a informace o každé z nich.
     */
    public static function getEmailTemplatesByType($idType) {
      return Database::queryMulti('
              SELECT `id_email_template`, blck_email_templates.id_incident_status, `id_email_lang`, `subject`, `body`,
              `lang`, `symbol`, `description`,
              `title`
              FROM `blck_email_templates`
              JOIN `blck_email_templates_langs`
              ON blck_email_templates.id_email_lang = blck_email_templates_langs.id_lang
              JOIN `blck_incidents_status`
              ON blck_incidents_status.id_incident_status = blck_email_templates.id_incident_status
              WHERE `id_incident_type` = ?
              ORDER BY blck_incidents_status.id_incident_status, `lang`'
        , $idType);
    }


    /**
     * Vrátí šablonu v požadované jazykové mutaci pro e-mail podle
     * konkrétního typu incidentu a podle konkrétního stavu.
     *
     * @param int $idType              ID typu bezpečnostního incidentu.
     * @param int $idStatus            ID stavu, pro nějž je šablona požadována.
     * @param string $lang             Jazyková mutace, v níž je šablona napsána.
     * @return mixed                   Pole obsahující informace o šabloně pro e-amil.
     */
    public static function getEmailTemplateByStatus($idType, $idStatus, $lang) {
      return Database::querySingle('
              SELECT `subject`, `body`
              FROM `blck_email_templates`
              JOIN `blck_email_templates_langs`
              ON blck_email_templates.id_email_lang = blck_email_templates_langs.id_lang
              WHERE `id_incident_type` = ?
              AND `id_incident_status` = ?
              AND `lang` = ?'
        , [$idType, $idStatus, $lang]);
    }


    /**
     * Vrátí možné jazykové mutace u konkrétní šablony podle zvoleného typu incidentu.
     *
     * @param int $idType              ID typu bezpečnostního incidentu.
     * @param bool $onlyLangNames      TRUE, pokud se mají vrátít pouze názvy jazykových mutací
     *                                 (výchozí volba) nebo (při FALSE) i další informace.
     * @return array                   Pole všech možných jazykových mutací pro danou šablonu.
     */
    public static function getEmailTemplatesLanguages($idType, $onlyLangNames = true) {
      $languages = [];

      $whereFilter = (!$onlyLangNames) ? ', `symbol`' : '';

      $result = Database::queryMulti('
        SELECT DISTINCT `lang`' . $whereFilter . '
        FROM `blck_email_templates`
        JOIN `blck_email_templates_langs`
        ON blck_email_templates.id_email_lang = blck_email_templates_langs.id_lang 
        WHERE `id_incident_type` = ?
        AND `body` IS NOT NULL
        AND `body` != ""
        ORDER BY `id_lang` DESC',
      $idType);

      if ($onlyLangNames) {
        foreach ($result as $lang) {
          $languages[] = $lang['lang'];
        }
      }
      else {
        $languages = $result;
      }

      return $languages;
    }


    /**
     * Vloží do databáze novou šablonu.
     */
    public function insertEmailTemplate() {
      $template = $this->makeEmailTemplate();

      Logger::info('Vložení nové šablony.', $template);

      Database::insert($this->dbTableName, $template);
    }


    /**
     * Vloží do databáze novou, prázdnou šablonu.
     *
     * @param int $idIncidentType      ID typu incidentu, ke kterému se šablona přidává.
     * @param int $idIncidentStatus    ID stavu incidentu, kterého se šablona týká.
     * @param int $idLang              ID jazykové mutace použité v šabloně.
     */
    public function insertEmptyEmailTemplate($idIncidentType, $idIncidentStatus, $idLang) {
      $template = $this->makeEmailTemplate();

      $template['id_incident_type'] = $idIncidentType;
      $template['id_incident_status'] = $idIncidentStatus;
      $template['id_email_lang'] = $idLang;

      Logger::info('Vložení nové, prázdné šablony.', $template);

      Database::insert($this->dbTableName, $template);
    }


    /**
     * Upraví zvolenou šablonu.
     *
     * @param int $id                  ID šablony
     */
    public function updateEmailTemplate($id) {
      $template = $this->makeEmailTemplate();

      Logger::info('Úprava existující šablony.', $template);

      Database::update(
        $this->dbTableName,
        $template,
        'WHERE `id_email_template` = ?',
        $id
      );
    }


    /**
     * Personalizuje a vrátí konkrétní obsah (např. předmět nebo obsah zprávy) nahrazením proměnných za skutečné
     * hodnoty (např. proměnnou %csirt_member_name% nahradí za jméno a příjmení uživatele z LDAP).
     *
     * @param string $content          Obsah (předmět/tělo zprávy), který má být personalizován.
     * @param string $networkName      Název počítačové sítě, v níž došlo k incidentu.
     * @param string $username         Uživatelské jméno, vůči kterému se budou z LDAP získávat informace.
     * @return string                  Personalizovaný obsah.
     */
    public static function getPersonalizedContent($content, $networkName, $username) {
      $ldap = new LdapModel();

      $userFullname = $ldap->getUserCNByUsername(get_email_part($username, 'username'));

      $csirtMemberName = $ldap->getUserCNByUsername(PermissionsModel::getUserName());
      $csirtName = CSIRT_NAME . ((!empty(CSIRT_ABRR)) ? ' (' . CSIRT_ABRR . ')' : '');

      $ldap->close();

      return str_replace(
        [VAR_INCIDENT_NETWORK, VAR_INCIDENT_USER, VAR_CSIRT_NAME, VAR_CSIRT_MEMBER_NAME],
        [$networkName, $userFullname, $csirtName, $csirtMemberName],
        $content
      );
    }


    /**
     * Zkontroluje uživatelský vstup (atributy třídy), který se bude zapisovat do databáze.
     *
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    public function validateData() {
      $this->isSubjectEmpty();
      $this->isSubjectTooLong();

      $this->isBodyEmpty();
      $this->isBodyTooLong();
    }


    /**
     * Ověří, zdali byl vyplněn předmět zprávy.
     *
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    private function isSubjectEmpty() {
      if (empty($this->subject)) {
        throw new UserError('Není vyplněn předmět zprávy.', MSG_ERROR);
      }
    }


    /**
     * Ověří, zdali zadaný předmět zprávy není příliš dlouhý.
     *
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    private function isSubjectTooLong() {
      if (mb_strlen($this->subject) > $this->inputsMaxLengths['subject']) {
        throw new UserError('Předmět zprávy je příliš dlouhý.', MSG_ERROR);
      }
    }


    /**
     * Ověří, zdali byl vyplněn obsah zprávy.
     *
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    private function isBodyEmpty() {
      if (empty($this->body)) {
        throw new UserError('Není vyplněn obsah zprávy.', MSG_ERROR);
      }
    }


    /**
     * Ověří, zdali zadaný obsah zprávy není příliš dlouhý.
     *
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    private function isBodyTooLong() {
      if (mb_strlen($this->body) > $this->inputsMaxLengths['body']) {
        throw new UserError('Obsah zprávy je příliš dlouhý.', MSG_ERROR);
      }
    }
  }
