<?php
  /**
   * Třída sloužící k získávání informací o typech bezpečnostních incidentů, k přidávání nových
   * typů incidentů, k úpravě těch existujících a k dalším souvisejícím operacím.
   *
   * @author Martin Šebela
   */
  class IncidentTypeModel extends FormModel {
    /**
     * @var string      Název tabulky v databázi, se kterou je instance svázána.
     */
    private $dbTableName = 'blck_incidents_types';

    /**
     * @var string      Název typu incidentu.
     */
    protected $name;

    /**
     * @var string      Popis typu incidentu.
     */
    protected $description;

    /**
     * @var string      Barva typu incidentu.
     */
    protected $color;


    /**
     * Připraví nový typ incidentu v závislosti na vyplněných datech a vrátí ho formou pole.
     *
     * @return array                   Pole obsahující data o typu incidentu.
     */
    public function makeIncidentType() {
      return [
        'name' => $this->name,
        'description' => $this->description,
        'color' => $this->color
      ];
    }


    /**
     * Uloží do instance a zároveň vrátí (z databáze) informace o zvoleném typu incidentu.
     *
     * @param int $id                  ID typu incidentu
     * @return array                   Pole obsahující informace o typu incidentu.
     */
    public function getIncidentType($id) {
      $this->dbRecordData = Database::querySingle('
              SELECT `id_incident_type`, `name`, `description`, `color`
              FROM `blck_incidents_types`
              WHERE `id_incident_type` = ?
              AND `visible` = 1'
      , $id);

      return $this->dbRecordData;
    }


    /**
     * Vrátí seznam všech typů bezpečnostních incidentů z databáze.
     *
     * @return mixed                   Pole typů bezpečnostních incidentů a informace o každém z nich.
     */
    public static function getIncidentTypes() {
      return Database::queryMulti('
              SELECT `id_incident_type`, `name`, `description`, `color`,
              `username`,
              DATE_FORMAT(blck_incidents_types.date_added, "%e. %c. %Y") AS date_added
              FROM `blck_incidents_types`
              JOIN `blck_users`
              ON blck_incidents_types.id_by_user = blck_users.id_user
              WHERE blck_incidents_types.visible = 1
      ');
    }


    /**
     * Vrátí název konkrétního typu incidentu.
     *
     * @param int $id                  ID typu incidentu
     * @return string                  Název typu incidentu.
     */
    public static function getNameIncidentType($id) {
      $name = '';

      $data = Database::querySingle('
              SELECT `name` FROM `blck_incidents_types` WHERE `id_incident_type` = ? AND `visible` = 1'
        , $id);

      if (isset($data['name'])) {
        $name = $data['name'];
      }

      return $name;
    }


    /**
     * Vrátí celkový počet evidovaných bezpečnostních incidentů,
     * u nichž je zvolen konkrétní typ incidentu.
     *
     * @param int $id                  ID typu bezpečnostního incidentu.
     * @return int                     Počet incidentů vyhovujících podmínce.
     */
    private function getCountOfUseIncidentType($id) {
      return Database::queryCount('
              SELECT COUNT(*)
              FROM `blck_incidents`
              WHERE `id_type` = ?
              AND `visible` = 1
      ', $id);
    }


    /**
     * Vloží do databáze nový typ incidentu.
     */
    public function insertIncidentType() {
      $incidentType = $this->makeIncidentType();

      $incidentType['id_by_user'] = $_SESSION['user']['id'];
      $incidentType['date_added'] = date('Y-m-d H:i:s');

      Logger::info('Vložení nového typu incidentu.', $incidentType);

      Database::insert($this->getDbTableName(), $incidentType);
      $idIncidentType = Database::getLastInsertId();

      $modelTemplates = new EmailTemplatesModel();
      $statuses = [3, 4, 5]; // Stavy, které budou u každé šablony předpřipravené.

      // Předpřipravit prázdné šablony pro e-maily pro všechny stavy (v češtině a angličtině).
      for ($i = 0; $i < count($statuses); $i++) {
        $modelTemplates->insertEmptyEmailTemplate($idIncidentType, $statuses[$i], 1);
        $modelTemplates->insertEmptyEmailTemplate($idIncidentType, $statuses[$i], 2);
      }
    }


    /**
     * Upraví zvolený typ bezpečnostního incidentu.
     *
     * @param int $id                  ID typu incidentu
     */
    public function updateIncidentType($id) {
      $incidentType = $this->makeIncidentType();

      Logger::info('Úprava existujícího typu incidentu.', $incidentType);

      Database::update(
        $this->getDbTableName(),
        $incidentType,
        'WHERE `id_incident_type` = ? AND `visible` = 1',
        $id
      );
    }


    /**
     * Odstraní (resp. deaktivuje) konkrétní typ bezpečnostního incidentu z databáze.
     *
     * @param int $id                  ID typu incidentu
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    public function deleteIncidentType($id) {
      if ($this->getCountOfUseIncidentType($id) != 0) {
        throw new UserError(
          'Nelze smazat typ incidentu, který je svázán s nějakým již evidovaným bezpečnostním incidentem.', MSG_ERROR
        );
      }

      $result = Database::update(
        'blck_incidents_types',
        ['visible' => 0],
        'WHERE `id_incident_type` = ? AND `visible` = 1',
        $id
      );

      if ($result == 0) {
        Logger::warning('Snaha o smazání neexistujícího typu incidentu.', $id);

        throw new UserError('Záznam vybraný ke smazání neexistuje.', MSG_ERROR);
      }

      Logger::info('Smazání existujícího typu incidentu.', $id);
    }


    /**
     * Zkontroluje uživatelský vstup (atributy třídy), který se bude zapisovat do databáze.
     *
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    public function validateData() {
      $this->isTypeNameEmpty();
      $this->isTypeNameTooLong();

      $this->isTypeDescriptionEmpty();
      $this->isTypeDescriptionTooLong();

      $this->isTypeColorValid();
    }


    /**
     * Ověří, zdali byl vyplněn název typu incidentu.
     *
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    private function isTypeNameEmpty() {
      if (empty($this->name)) {
        throw new UserError('Není vyplněn název typu incidentu.', MSG_ERROR);
      }
    }


    /**
     * Ověří, zdali zadaný název typu incidentu není příliš dlouhý.
     *
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    private function isTypeNameTooLong() {
      if (mb_strlen($this->name) > $this->inputsMaxLengths['name']) {
        throw new UserError('Název typu incidentu je příliš dlouhý.', MSG_ERROR);
      }
    }


    /**
     * Ověří, zdali byl vyplněn popis typu incidentu.
     *
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    private function isTypeDescriptionEmpty() {
      if (empty($this->description)) {
        throw new UserError('Není vyplněn popis typu incidentu.', MSG_ERROR);
      }
    }


    /**
     * Ověří, zdali zadaný popis typu incidentu není příliš dlouhý.
     *
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    private function isTypeDescriptionTooLong() {
      if (mb_strlen($this->description) > $this->inputsMaxLengths['description']) {
        throw new UserError('Popis typu incidentu je příliš dlouhý.', MSG_ERROR);
      }
    }


    /**
     * Ověří, zdali je vybraná barva typu incidentu validní.
     *
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    private function isTypeColorValid() {
      if (mb_strlen($this->color) > $this->inputsMaxLengths['color'] || mb_strlen($this->color) != 7) {
        throw new UserError('Barva typu incidentu je v nesprávném formátu.', MSG_ERROR);
      }
    }
  }
