<?php
  /**
   * Třída řešící přihlašování, registraci, oprávnění uživatelů v systému a získávání informací
   * o přihlášeném uživateli.
   *
   * @author Martin Šebela
   */
  class PermissionsModel {
    /**
     * Registruje do systému nového uživatele na základě jeho uživatelského jména,
     * které se před samotnou registrací ověří v LDAPu.
     *
     * @param string $username         Uživatelské jméno
     */
    private function register($username) {
      $newUser = new UsersModel();
      $ldap = new LdapModel();

      // Získání e-mailu uživatele z LDAP.
      $newUser->email = $ldap->getEmailByUsername($username);
      $ldap->close();

      // Pokud se podařilo z LDAPu získat e-mail, dojde k registraci nového uživatele.
      if (!empty($newUser->email)) {
        Logger::info('Registrace nového uživatele.', $username);

        $newUser->insertUser();
        $this->login($username);
      }
      else {
        Logger::error('Při registraci se nepodařilo načíst e-mail uživatele z LDAP.', $username);

        // Pokud se nepodaří načíst e-mail z LDAPu, přesměrovat uživatele na úvodní stránku systému.
        header('Location: ' . WEB_URL);
        exit();
      }
    }


    /**
     * Přihlásí konkrétního uživatele do systému.
     *
     * @param string $username         Uživatelské jméno
     */
    public function login($username) {
      // Ověření, zdali uživatelské jméno není prázdné, tzn. zdali Shibboleth (SSO) něco předal.
      if (empty($username)) {
        Logger::error('Při přihlašování se nepodařilo načíst uživatelské jméno uživatele ze Shibboleth.');

        header('Location: ' . WEB_URL);
        exit();
      }

      $user = UsersModel::getUserByUsername($username);

      // Pokud je již uživatel registrován...
      if (!empty($user)) {
        $_SESSION['user']['id'] = $user['id_user'];
        $_SESSION['user']['username'] = $user['username'];

        // Vytvoření unikátního CSRF tokenu pro jednu relaci přihlášení.
        $_SESSION['csrf_token'] = $this->generateCsrfToken($user['id_user']);

        session_regenerate_id();

        // Zaznamenat přihlášení uživatele do databáze.
        $this->logLogin($user['id_user']);
      }
      else {
        $this->register($username);
      }
    }


    /**
     * Vytvoří v databázi nový záznam o úspěšném přihlášení uživatele do systému.
     *
     * @param int $idUser              ID uživatele
     */
    private function logLogin($idUser) {
      Logger::info('Přihlášení uživatele do systému.', $idUser);

      $record = [
        'id_user' => $idUser,
        'login_datetime' => date('Y-m-d H:i:s'),
        'ip' => get_client_ip()
      ];

      Database::insert('blck_users_login_log', $record);
    }


    /**
     * Vrátí počet přihlášení konkrétního uživatele do systému.
     *
     * @param $idUser                  ID uživatele
     * @return mixed                   Celkový počet přihlášení
     */
    private function getCountOfUserLogins($idUser) {
      return Database::queryCount('
              SELECT COUNT(*)
              FROM `blck_users_login_log`
              WHERE `id_user` = ?
      ', $idUser);
    }


    /**
     * Vygeneruje a vrátí unikátní CSRF token pro konkrétního uživatele.
     *
     * @param int $idUser              ID uživatele
     * @return string                  Vygenerovaný CSRF token
     */
    private function generateCsrfToken($idUser) {
      return hash_hmac('sha512', microtime() . $idUser, CSRF_KEY);
    }


    /**
     * Vrátí CSRF token.
     *
     * @return mixed|null              CSRF token nebo NULL
     */
    public static function getCsrfToken() {
      if (isset($_SESSION['csrf_token'])) {
        return $_SESSION['csrf_token'];
      }

      return null;
    }


    /**
     * Odhlásí právě přihlášeného uživatele ze systému.
     */
    public static function logout() {
      unset($_SESSION['user']);
    }


    /**
     * Vrátí ID právě přihlášeného uživatele.
     *
     * @return int|null                ID uživatele nebo NULL
     */
    public static function getUserId() {
      if (isset($_SESSION['user']['id'])) {
        return $_SESSION['user']['id'];
      }

      return null;
    }


    /**
     * Vrátí uživatelské jméno právě přihlášeného uživatele.
     *
     * @return string|null                Uživatelské jméno uživatele nebo NULL
     */
    public static function getUserName() {
      if (isset($_SESSION['user']['username'])) {
        return $_SESSION['user']['username'];
      }

      return null;
    }
  }
