<?php
  /**
   * Třída zabezpečující komunikaci se systémem pro správu požadavků RT (Request Tracker)
   * pomocí endpointů, které poskytuje rozhraní REST API.
   *
   * Částečná dokumentace k endpointům REST API u systému RT:
   * https://rt-wiki.bestpractical.com/wiki/REST
   *
   * @author Martin Šebela
   */
  class TicketingSystem {
    /**
     * URL služby pro volání RESP API.
     */
    const serviceBaseUrl = 'https://rt.zcu.cz/rt/REST/1.0/';


    /**
     * Vytvoří v systému RT nový lístek a vrátí jeho ID, které bylo lístku v RT systému přiděleno.
     *
     * @param string $requestor        Žadatel lístku
     * @param string $subject          Předmět lístku
     * @param string $content          Obsah lístku
     * @return int|null                ID nově vytvořeného lístku nebo NULL při neúspěchu.
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    public static function addNewTicket($requestor, $subject, $content) {
      $newTicketID = null;

      $data = [
        'id' => 'ticket/new',
        'Queue' => RT_DEFAULT_QUEUE,
        'Requestor' => $requestor,
        'Subject' => $subject,
        'Text' => self::getMutlilineText($content)
      ];

      $response = self::callEndpoint('ticket/new', $data);

      // Získání přiděleného ID nově vytvořeného lístku.
      preg_match('/Ticket ([0-9]*) created/', $response, $matches);

      if (is_array($matches) && is_numeric(end($matches))) {
        $newTicketID = end($matches);
      }

      return $newTicketID;
    }


    /**
     * Vloží odpověď do již existujícího lístku.
     *
     * @param int $id                  Identifikátor lístku, ke kterému má být odpověď vložena
     * @param string $reply            Odpověď do lístku
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    public static function replyTicket($id, $reply) {
      $data = [
        'id' => $id,
        'Action' => 'correspond',
        'Text' => self::getMutlilineText($reply)
      ];

      if (is_numeric($id)) {
        $response = self::callEndpoint('ticket/' . $id . '/comment', $data);

        //print_r($response);
      }
    }


    /**
     * Vloží komentář do již existujícího lístku.
     *
     * @param int $id                  Identifikátor lístku, ke kterému má být odpověď vložena
     * @param string $comment          Komentář k lístku
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    public static function commentTicket($id, $comment) {
      $data = [
        'id' => $id,
        'Action' => 'comment',
        'Text' => self::getMutlilineText($comment)
      ];

      if (is_numeric($id)) {
        $response = self::callEndpoint('ticket/' . $id . '/comment', $data);

        //print_r($response);
      }
    }


    /**
     * Změní stav u již existujícího lístku.
     * Možné stavy pravděpodobně jsou: open/new/stalled/resolved/rejected/feedback/deleted
     *
     * @param int $id                  Identifikátor lístku, u něhož se má stav změnit
     * @param string $status           Název nového stavu lístku
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    public static function changeTicketStatus($id, $status) {
      $data = [
        'id' => $id,
        'status' => $status
      ];

      if (is_numeric($id)) {
        $response = self::callEndpoint('ticket/' . $id . '/edit', $data);

        //print_r($response);
      }
    }


    /**
     * Přidá vazbu (resp. závislost) u existujícího lístku na jiný, již existující lístek.
     *
     * @param int $childTicketId       Číslo lístku
     * @param int $parentTicketId      Číslo rodičovského lístku
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    public static function addDependsToTicket($childTicketId, $parentTicketId) {
      $data = [
        'DependsOn' => $childTicketId
      ];

      if (is_numeric($childTicketId) && is_numeric($parentTicketId)) {
        $response = self::callEndpoint('ticket/' . $parentTicketId . '/links', $data);

        //print_r($response);
      }
    }


    /**
     * Vrátí metadata o již existujícím lístku.
     *
     * @param int $id                  Číslo lístku
     * @return array|null              Metadata o lístku nebo NULL
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    public static function getTicketMetadata($id) {
      $response = null;

      if (is_numeric($id)) {
        $response = self::callEndpoint('ticket/' . $id . '/show');
        $response = self::parseResponse($response);

        //print_r($response);
      }

      return $response;
    }


    /**
     * Připraví odpověď pro REST API tak, jak vyžaduje RT (tj. data uložená jako string řádek po řádku).
     *
     * @param array $data              Data formou pole, která se převedou do podoby vyžadované systémem RT
     * @return string                  Data transforomovaná do podoby požadovaná systémem RT
     */
    private static function prepareDataToServer($data) {
      $response = '';

      if (is_array($data)) {
        foreach ($data as $key => $value) {
          $response .= $key . ': ' . $value . "\n";
        }
      }

      return $response;
    }


    /**
     * Zavolá konkrétní endpoint REST API.
     *
     * @param string $endpoint         Název endpointu
     * @param array|null $data         Data předané službě nebo NULL, pokud žádná data nejou
     * @return bool|string|null        Výsledek akce (obsah stránky)
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    private static function callEndpoint($endpoint, $data = null) {
      $endpoint = self::serviceBaseUrl . $endpoint;

      if ($data != null) {
        // RT požaduje jako vstup proměnnou s názvem "content",
        // kde jsou na jednotlivých řádcích předaná data.
        $data = ['content' => self::prepareDataToServer($data)];
      }

      return process_sso($endpoint, $data);
    }


    /**
     * Rozdělí odpověď ze serveru na prvky pole a výsledné pole vrátí.
     * viz: https://github.com/dersam/RTPHPLib/blob/master/RequestTracker.php
     *
     * @param string $response         Odpověď ze serveru.
     * @param string $delimiter        Použitý oddělovač (mezi klíčem a hodnotou).
     * @return array                   Odpověď ze serveru formou pole.
     */
    private static function parseResponse($response, $delimiter = ':') {
      $response = explode(chr(10), $response);
      $response = self::cleanResponseBody($response);

      return self::parseResponseBody($response);
    }


    /**
     * Vyčištění odpovědi ze serveru.
     * viz https://github.com/dersam/RTPHPLib/blob/master/RequestTracker.php
     *
     * @param array $response          Pole, která se má vyčistit od speciálních znaků (prázdné řádky apod.).
     * @return array                   Vyčištěné pole.
     */
    private static function cleanResponseBody(array $response) {
      array_shift($response); // skip RT status response
      array_shift($response); // skip blank line
      array_pop($response);   // remove empty blank line in the end

      return $response;
    }


    /**
     * Rozdělí tělo e-mailu na jednotlivé prvky pole.
     * viz: https://github.com/dersam/RTPHPLib/blob/master/RequestTracker.php
     *
     * @param array $response          Odpověď ze serveru.
     * @param string $delimiter        Použitý oddělovač (mezi klíčem a hodnotou).
     * @return array                   Odpověď ze serveru formou pole.
     */
    private static function parseResponseBody(array $response, $delimiter = ':') {
      $responseArray = array();
      $lastkey = null;

      foreach ($response as $line) {
        // RT will always preface a multiline with the length of the last key + length of $delimiter + one space)
        if (!is_null($lastkey) && preg_match('/^\s{' . ( strlen($lastkey) + strlen($delimiter) + 1 ) . '}(.*)$/', $line, $matches)) {
          $responseArray[$lastkey] .= "\n" . $matches[1];
        }
        elseif (!is_null($lastkey) && strlen($line) == 0) {
          $lastkey = null;
        }
        elseif (preg_match('/^#/', $line, $matches)) {
          $responseArray[$line] = '';
        }
        elseif (preg_match('/^([a-zA-Z0-9]+|CF\.{[^}]+})' . $delimiter . '\s(.*)$/', $line, $matches)) {
          $lastkey = $matches[1];
          $responseArray[$lastkey] = $matches[2];
        }
      }

      return $responseArray;
    }


    /**
     * Připraví víceřádkový (multiline) text pro potřeby RT, které vyžaduje, aby
     * nové řádky začínaly prázdným znakem.
     *
     * @param string $content          Obsah, který se má připravit pro vkládání do RT.
     * @return string                  Připravený obsah pro vkládání do RT.
     */
    private static function getMutlilineText($content) {
      return str_replace("\n", "\n ", $content);
    }
  }