<?php
  /**
   * Blockátor pro VPN síť. Obsahuje funkce pro vyhledání uživatele z logů,
   * funkce pro blokaci a odblokování konkrétního uživatele.
   *
   * @author Martin Šebela
   */
  class VPNBlocker extends Blocker implements IBlocker {
    /**
     * @var string      Název (označení) sítě pro textové výpisy.
     */
    protected $networkName = 'VPN';

    /**
     * @var string      Vzor na základě něhož bude rozhodnuto, že zařízení spadá do rozsahu této sítě.
     */
    private $networkPatternHostname = 'vpn';

    /**
     * @var string      Cesta k bash skriptu, který slouží k (od)blokaci uživatelů.
     */
    private $scriptRadiusFilepath = CORE_DOCUMENT_ROOT . '/' . CORE_DIR_BLOCKERS . '/remoteServers.sh';

    /**
     * @var string      Cesta v souborovém systému, kde jsou uloženy logy.
     */
    private $logBaseFilepath = '/mnt/data/asavpn/';


    /**
     * Vrátí, zdali hostname spadá do rozsahu sítě.
     *
     * @param string $hostname         Hostname zařízení
     * @return bool                    TRUE pokud spadá do oblasti sítě, jinak FALSE.
     */
    public function isFromThisNetwork($hostname) {
      if (strpos($hostname, mb_strtolower($this->networkPatternHostname)) !== false) {
        return true;
      }

      return false;
    }


    /**
     * Zařídí vyhledávání záznamů v logu o dané IP adrese v určitém datu a nalezená data vrátí.
     *
     * @param string $ipAddress        IP adresa
     * @param string $macAddress       MAC adresa (nemá význam, nebude použita).
     * @param string $datetime         Datum a čas
     * @return array                   Nalezená data v logu
     */
    public function find($ipAddress, $macAddress, $datetime) {
      $foundData = $this->searchInLog($ipAddress, $datetime);

      return [
        'logs' => $foundData['logs'],
        'macAddresses' => [],
        'hostnames' => [],
        'users' => $foundData['users']
      ];
    }


    /**
     * Zablokuje konkrétního uživatele v síti VPN.
     *
     * @param BlockedIdentity $identity Identita, která se má zablokovat.
     * @return bool                     TRUE při úspěšné blokaci.
     * @throws UserError                Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    public function block(BlockedIdentity $identity) {
      $result = false;

      if (is_valid_username($identity->getUsername())) {
        $username = escapeshellcmd($identity->getUsername());
        shell_exec($this->scriptRadiusFilepath . ' vpn block "' . $username . '"'); // 2>&1 (pro ladění)

        $this->addNoteToUsersDatabase($identity->getUsername(), $this->getBlockedMessage($identity));

        $result = true;
      }

      return $result;
    }


    /**
     * Odblokuje konkrétního uživatele v síti VPN.
     *
     * @param BlockedIdentity $identity Identita, která se má odblokovat.
     * @return bool                     TRUE při úspěšném odblokování.
     * @throws UserError                Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    public function unblock(BlockedIdentity $identity) {
      $result = false;

      if (is_valid_username($identity->getUsername())) {
        $username = escapeshellcmd($identity->getUsername());
        shell_exec($this->scriptRadiusFilepath . ' vpn unblock "' . $username . '"');

        $this->addNoteToUsersDatabase($identity->getUsername(), $this->getUnblockedMessage($identity));

        $result = true;
      }

      return $result;
    }


    /**
     * Vyhledá záznamy v konkrétním logu o dané IP adrese v inkriminovaném datu.
     *
     * @param string $ipAddress        IP adresa (IPv4 nebo IPv6)
     * @param string $datetime         Datum a čas
     * @return array                   Záznamy nalezené v logu
     */
    private function searchInLog($ipAddress, $datetime) {
      $logRecords = [];
      $foundUsers = [];

      $date = get_date_parts($datetime);

      // Cesta k logu, ve kterém se bude vyhledávat.
      $file = $this->logBaseFilepath . $date['year'] . '/' . $date['month'] . '/log.' . $date['year'] . $date['month'] . $date['day'];

      $contents = file_get_contents($file);

      // Regulární výraz pro vyhledávání v logu.
      if (filter_var($ipAddress, FILTER_VALIDATE_IP, FILTER_FLAG_IPV6)) {
        // Vyhledávání podle IPv6.
        $regexp = '(.*) Group (.*) User <(.*)> IP <(.*)> IPv4 Address <(.*)> IPv6 address <' . $ipAddress . '> assigned to (.*)';
      }
      else {
        // Vyhledávání podle IPv4.
        $regexp = '(.*) Group (.*) User <(.*)> IP <(.*)> IPv4 Address <' . $ipAddress . '> IPv6 address <(.*)> assigned to (.*)';
      }

      // Nalezení záznamů s hledanou IP adresou v inkriminovaném datu.
      if (preg_match_all('/' . $regexp . '/', $contents, $matches)) {
        $logRecords = $matches[0];
        $foundUsers = $matches[3];
      }

      $foundUsers = remove_duplicates_in_array($foundUsers);

      return [
        'logs' => $logRecords,
        'users' => $foundUsers
      ];
    }
  }