<?php
  /**
   * Blockátor pro zařízení pevně připojená do sítě WEBnet.
   *
   * @author Martin Šebela
   */
  class WiredBlocker extends Blocker implements IBlocker {
    /**
     * @var string      Název (označení) sítě pro textové výpisy.
     */
    protected $networkName = 'pevná síť';

    /**
     * @var string      Vzor na základě něhož bude rozhodnuto, že zařízení spadá do rozsahu této sítě.
     */
    private $networkPatternHostname = '.zcu.cz';

    /**
     * @var string      URL volané služby, kde dochází k blokaci nebo odblokování.
     */
    private $serviceBaseUrl = 'https://netspy.civ.zcu.cz/';

    /**
     * @var string      Cesta v souborovém systému, kde jsou uloženy DHCP logy.
     */
    private $logBaseFilepath = '/mnt/data/dhcpd-zcu/';


    /**
     * Vrátí, zdali hostname spadá do rozsahu sítě.
     *
     * @param string $hostname         Hostname zařízení
     * @return bool                    TRUE pokud spadá do oblasti sítě, jinak FALSE.
     */
    public function isFromThisNetwork($hostname) {
      if (strpos($hostname, mb_strtolower($this->networkPatternHostname)) !== false) {
        return true;
      }

      return false;
    }


    /**
     * Vyhledá detailnější informace o konkrétním zařízení a především informace
     * o lokálním správci zařízení a informace předá formou pole.
     *
     * @param string $ipAddress        IP adresa
     * @param string $macAddress       MAC adresa
     * @param string $datetime         Datum a čas
     * @return array                   Nalezená data v logu
     */
    public function find($ipAddress, $macAddress, $datetime) {
      $foundData = $this->searchInDhcpLog($ipAddress, $macAddress, $datetime);;
      $foundData['users'] = [];

      foreach ($foundData['hostnames'] as $hostname) {
        // Zjištění lokálních správců pro každé zařízení.
        $foundContacts = $this->getContacts($hostname);

        foreach ($foundContacts as $contact) {
          $foundData['users'][] = $contact;
        }

        // Pokud je více lokálních správců, pak zduplikovat informace
        // o zařízení pro každého lokálního správce.
        for ($i = 1; $i < count($foundContacts); $i++) {
          $foundData['macAddresses'][] = $foundData['macAddresses'][0];
          $foundData['hostnames'][] = $hostname;
        }
      }

      return [
        'logs' => $foundData['logs'],
        'macAddresses' => $foundData['macAddresses'],
        'hostnames' => $foundData['hostnames'],
        'users' => $foundData['users']
      ];
    }


    /**
     * Zablokuje konkrétní zařízení pevně připojené do sítě WEBnet.
     *
     * @param BlockedIdentity $identity Identita, která se má zablokovat.
     * @throws UserError                Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    public function block(BlockedIdentity $identity) {
      $result = false;

      // Získání přehledu z externí služby NetSpy, kde jde uvedeno, kdy bylo zařízení připojeno do switche.
      $deviceSummary = $this->getDeviceSummary($identity->getIpAddress(), $identity->getMacAddress());

      // Příklad: i=147.228.1.112&amp;eid=(.*)&amp;action=block&amp;action_type=1">SET PORT DOWN
      $regexp = 'i=' . $identity->getIpAddress() . '&amp;eid=(.*)&amp;action=block&amp;action_type=1">SET PORT DOWN';
      $deviceTimestamps = [];

      // Nalezení všech výskytů, kde je v NetSpy tlačítko "SET PORT DOWN" pro dané zařízení.
      if (preg_match_all('/' . $regexp . '/', $deviceSummary, $matches)) {
        $deviceTimestamps = $matches[1];
      }

      // Získání největšího timestamp - tedy posledního data, kdy bylo zařízení připojeno do sítě.
      $maxTimestamp = max($deviceTimestamps);

      // Zaslat externí službě NetSpy požadavek na shození portu na switchi u zařízení,
      // které bylo v daný čas připojeno do sítě.
      if (!empty($deviceTimestamps) && $maxTimestamp != null) {
        $arguments = [
          'p' => 'mac_ip_detail',
          'm' => $identity->getMacAddress(),
          'i' => $identity->getIpAddress(),
          'eid' => $maxTimestamp,
          'action' => 'block',
          'action_type' => 1
        ];

        // Samotné zablokování a shození portu na switchi u daného zařízení.
        // Příklad: https://netspy.civ.zcu.cz/index.php?p=mac_ip_detail&m=00:25:64:98:E6:E4&i=147.228.1.112&eid=309697530&action=block&action_type=1
        $url = $this->serviceBaseUrl . '?' . http_build_query($arguments);
        $response = process_sso($url);

        // NetSpy při úspěšném zablokování vypisuje do stránky zprávu "PORT SHUTDOWN: ...",
        // pokud je tedy ve výstupu stránky, pak shození portu proběhlo úspěšně.
        if (strpos($response, 'PORT SHUTDOWN:') !== false) {
          $result = true;
        }
      }

      return $result;
    }


    /**
     * Odblokuje konkrétní zařízení pevně připojené do sítě WEBnet.
     *
     * @param BlockedIdentity $identity Identita, která se má odblokovat.
     * @throws UserError                Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    public function unblock(BlockedIdentity $identity) {
      $result = false;

      // Získání přehledu, kdy bylo zařízení připojeno do sítě.
      $deviceSummary = $this->getDeviceSummary($identity->getIpAddress(), $identity->getMacAddress());

      // Příklad: i=147.228.1.112&amp;bid=(.*)&amp;action=unblock&amp;action_type=1">SET PORT UP
      $regexp = 'i=' . $identity->getIpAddress() . '&amp;bid=(.*)&amp;action=unblock&amp;action_type=1">SET PORT UP';
      $deviceBlocationIds = [];

      // Nalezení všech výskytů, kde je v NetSpy tlačítko "SET PORT UP" pro dané zařízení.
      if (preg_match_all('/' . $regexp . '/', $deviceSummary, $matches)) {
        $deviceBlocationIds = $matches[1];
      }

      // Uvažovat první ID blokace, které bylo nalezeno na stránce.
      $blocationId = $deviceBlocationIds[0];

      // Zaslat externí službě NetSpy požadavek na aktivaci portu na switchi u zablokovaného zařízení.
      if (!empty($deviceBlocationIds) && $blocationId != null) {
        $arguments = [
          'p' => 'mac_ip_detail',
          'm' => $identity->getMacAddress(),
          'i' => $identity->getIpAddress(),
          'bid' => $blocationId,
          'action' => 'unblock',
          'action_type' => 1
        ];

        // Odblokování a nahození portu na switchi u daného zařízení.
        $url = $this->serviceBaseUrl . '?' . http_build_query($arguments);
        $response = process_sso($url);

        // Pokud se v odpovědi ze služby NetSpy objeví "PORT STATE SET UP",
        // pak to znamená, že port byl na switchi úspěšně nahozen.
        if (strpos($response, 'PORT STATE SET UP:') !== false) {
          $result = true;
        }
      }

      return $result;
    }


    /**
     * Vyhledá záznamy v DHCP logu o dané IP adrese a v inkriminovaném datu a čase.
     *
     * @param string $ipAddress        IP adresa
     * @param string $macAddress       MAC adresa zařízení (nepovinné)
     * @param string $datetime         Datum a čas
     * @return array                   Záznamy nalezené v logu
     */
    private function searchInDhcpLog($ipAddress, $macAddress, $datetime) {
      $logRecords = [];
      $macAddresses = [];
      $hostnames = [];

      $date = get_date_parts($datetime);

      // Cesta k DHCP logu.
      $file = $this->logBaseFilepath . $date['year'] . '/' . $date['month'] . '/dhcpd.log.' . $date['year'] . $date['month'] . $date['day'];

      $contents = file_get_contents($file);

      // Regulární výraz pro vyhledávání v logu.
      if ($macAddress != null) {
        // Vyhledávání podle IP i MAC adresy.
        $regexp = '(.*) DHCPACK on ' . $ipAddress . ' to ' . $macAddress . ' via (.*)';
      }
      else {
        // Vyhledávání pouze podle IP adresy.
        $regexp = '(.*) DHCPACK on ' . $ipAddress . ' to (.*) via (.*)';
      }

      // Nalezení záznamů DHCPACK s hledanou IP adresou v inkriminovaném datu.
      if (preg_match_all('/' . $regexp . '/', $contents, $matches)) {
        $logRecords[] = '[header]--------------------------- DHCP log ---------------------------[/header]';
        $logRecords = array_merge($logRecords, $matches[0]);

        // Pokud se má vyhledávat i MAC adresa, přidat ji do výstupu...
        if ($macAddress == null) {
          $macAddresses = $matches[2];
        }
        else {
          // Jinak použít MAC adresu, která byla již známá...
          $macAddresses[] = $macAddress;
        }
      }

      // Odstranění duplicit.
      $macAddresses = remove_duplicates_in_array($macAddresses);

      // Doplnění hostnames.
      for ($i = 0; $i < count($macAddresses); $i++) {
        $hostnames[] = gethostbyaddr($ipAddress);
      }

      return [
        'logs' => $logRecords,
        'macAddresses' => $macAddresses,
        'hostnames' => $hostnames,
      ];
    }


    /**
     * Vrátí přehled o zařízení připojeného do sítě WEBnet z nástroje NetSpy.
     *
     * @param string $ipAddress        IP adresa zařízení
     * @param string $macAddress       MAC adresa zařízení
     * @return string                  Obsah stránky vrácený z nástroje NetSpy.
     * @throws UserError               Výjimka obsahující textovou informaci o chybě pro uživatele.
     */
    private function getDeviceSummary($ipAddress, $macAddress) {
      $arguments = [
        'p' => 'mac_ip_detail',
        'm' => $macAddress,
        'i' => $ipAddress
      ];

      // Otevření přehledu o připojení s daným zařízením v NetSpy.
      // Příklad: https://netspy.civ.zcu.cz/index.php?p=mac_ip_detail&m=00:25:64:98:E6:E4&i=147.228.1.112
      $url = $this->serviceBaseUrl . '?' . http_build_query($arguments);
      $response = process_sso($url);

      if (empty($response)) {
        throw new UserError('Nepodařilo se vytvořit spojení se vzdálenou službou NetSpy při získávání informací o zařízení.', MSG_ERROR);
      }

      return $response;
    }


    /**
     * Vrátí informace o odpovědném lokálním správci (popř. správcích, pokud jich je více),
     * který má dle hostname zařízení na starost.
     *
     * @param string $hostname         Hostname zařízení
     * @return array|null              Informace o lokálním správci (správcích) formou pole nebo NULL.
     */
    private function getContacts($hostname) {
      $contacts = [];

      // Pokud hostname zařízení vyhovuje reg. výrazu, pak vrátit odpovědného lokálního správce.
      // Pozn.: pro každé pracoviště jich může být více, proto se ukládají do pole, které se následně vrátí.

      echo $hostname . '<br>';

      // příklad: uc307p02-kiv.fav.zcu.cz
      if (preg_match('/-(.*)\.(.*)\.zcu\.cz/', $hostname, $matches)) {
        $contacts = array_merge($contacts, $this->getContactByAttribute($matches[1], 'department_name'));
      }

      // příklad: cd312p51-ps.ps.fek.zcu.cz (nyní ale hledat podle názvu fakulty, resp. budovy)
      if (empty($contacts) && preg_match('/-(.*)\.(.*)\.zcu\.cz/', $hostname, $matches)) {
        $contacts = array_merge($contacts, $this->getContactByAttribute(end($matches), 'department_building'));
      }

      // příklad: wiener.kky.zcu.cz
      if (empty($contacts) && preg_match('/\.(.*)\.zcu\.cz/', $hostname, $matches)) {
        $contacts = array_merge($contacts, $this->getContactByAttribute($matches[1], 'department_name'));
      }

      // příklad: proteus.fav.zcu.cz
      if (empty($contacts) && preg_match('/\.(.*)\.zcu\.cz/', $hostname, $matches)) {
        $contacts = array_merge($contacts, $this->getContactByAttribute($matches[1], 'department_building'));
      }

      return (!empty($contacts)) ? remove_duplicates_in_array($contacts) : null;
    }


    /**
     * Vrátí pole lokálních správců, kteří vyhovují zadaným parametrům.
     *
     * @param string $value            Hodnota, která se bude porovnávat.
     * @param string $attribute        Sloupec z databáze, jehož hodnota se bude porovnávat s prvním parametrem.
     * @return array                   Pole lokálních správců.
     */
    private function getContactByAttribute($value, $attribute) {
      $selectedContacts = [];

      $contacts = Database::queryMulti(
        'SELECT `email`, `department_building`, `department_name`
                FROM `blck_departmens_contacts`
                WHERE `visible` = 1');

      foreach ($contacts as $contact) {
        if (strcmp(mb_strtolower($value), mb_strtolower($contact[$attribute])) == 0) {
          $selectedContacts[] = get_email_part($contact['email'], 'username');
        }
      }

      return $selectedContacts;
    }
  }